@@ -338,10 +338,20 @@ impl FrameData {
     pub fn write_into(&self, write: &mut impl std::io::Write) -> anyhow::Result<()> {
         use bincode::Options as _;
         let serialized = bincode::options().serialize(self)?;
-        let compressed = lz4_flex::compress_prepend_size(&serialized);
-        write.write_all(b"PFD0")?;
-        write.write_all(&(compressed.len() as u32).to_le_bytes())?;
-        write.write_all(&compressed)?;
+
+        if false {
+            let compressed = lz4_flex::compress_prepend_size(&serialized);
+            write.write_all(b"PFD0")?;
+            write.write_all(&(compressed.len() as u32).to_le_bytes())?;
+            write.write_all(&compressed)?;
+        } else {
+            // zstd cuts sizes in half compared to lz4_flex
+            let level = 3;
+            let compressed = zstd::encode_all(std::io::Cursor::new(&serialized), level)?;
+            write.write_all(b"PFD1")?;
+            write.write_all(&(compressed.len() as u32).to_le_bytes())?;
+            write.write_all(&compressed)?;
+        }
         Ok(())
     }
 
@@ -364,21 +374,39 @@ impl FrameData {
 
         if header == [0_u8; 4] {
             Ok(None) // end-of-stream sentinel.
-        } else if &header == b"PFD0" {
-            let mut compressed_length = [0_u8; 4];
-            read.read_exact(&mut compressed_length)?;
-            let mut compressed = vec![0_u8; u32::from_le_bytes(compressed_length) as usize];
-            read.read_exact(&mut compressed)?;
+        } else if header.starts_with(b"PFD") {
+            if &header == b"PFD0" {
+                let mut compressed_length = [0_u8; 4];
+                read.read_exact(&mut compressed_length)?;
+                let mut compressed = vec![0_u8; u32::from_le_bytes(compressed_length) as usize];
+                read.read_exact(&mut compressed)?;
 
-            let serialized =
-                lz4_flex::decompress_size_prepended(&compressed).context("lz4 decompress")?;
+                let serialized =
+                    lz4_flex::decompress_size_prepended(&compressed).context("lz4 decompress")?;
 
-            use bincode::Options as _;
-            Ok(Some(
-                bincode::options()
-                    .deserialize(&serialized)
-                    .context("bincode deserialize")?,
-            ))
+                use bincode::Options as _;
+                Ok(Some(
+                    bincode::options()
+                        .deserialize(&serialized)
+                        .context("bincode deserialize")?,
+                ))
+            } else if &header == b"PFD1" {
+                let mut compressed_length = [0_u8; 4];
+                read.read_exact(&mut compressed_length)?;
+                let mut compressed = vec![0_u8; u32::from_le_bytes(compressed_length) as usize];
+                read.read_exact(&mut compressed)?;
+
+                let serialized = zstd::decode_all(&compressed[..]).context("zstd decompress")?;
+
+                use bincode::Options as _;
+                Ok(Some(
+                    bincode::options()
+                        .deserialize(&serialized)
+                        .context("bincode deserialize")?,
+                ))
+            } else {
+                anyhow::bail!("Failed to decode: this data is newer than this reader. Please update your puffin version!");
+            }
         } else {
             // Old packet without magic header
             let mut bytes = vec![0_u8; u32::from_le_bytes(header) as usize];
@@ -482,6 +510,7 @@ impl ThreadProfiler {
     }
 
     /// Do something with the thread local `ThreadProfiler`
+    #[inline]
     pub fn call<R>(f: impl Fn(&mut Self) -> R) -> R {
         thread_local! {
             pub static THREAD_PROFILER: std::cell::RefCell<ThreadProfiler> = Default::default();
@@ -738,6 +767,7 @@ impl GlobalProfiler {
 
 /// Returns monotonically increasing nanosecond count.
 /// It is undefined when `now_ns()=0` is.
+#[inline]
 pub fn now_ns() -> NanoSecond {
     // This can maybe be optimized
     use once_cell::sync::Lazy;
@@ -765,6 +795,7 @@ impl ProfilerScope {
     /// The `id` doesn't need to be static, but it should be unchanging,
     /// and this is a good way to enforce it.
     /// `data` can be changing, i.e. a name of a mesh or a texture.
+    #[inline]
     pub fn new(id: &'static str, location: &str, data: impl AsRef<str>) -> Self {
         Self {
             start_stream_offset: ThreadProfiler::call(|tp| {
@@ -776,12 +807,14 @@ impl ProfilerScope {
 }
 
 impl Drop for ProfilerScope {
+    #[inline]
     fn drop(&mut self) {
         ThreadProfiler::call(|tp| tp.end_scope(self.start_stream_offset));
     }
 }
 
 #[doc(hidden)]
+#[inline(always)]
 pub fn type_name_of<T>(_: T) -> &'static str {
     std::any::type_name::<T>()
 }
@@ -799,6 +832,7 @@ macro_rules! current_function_name {
 }
 
 #[doc(hidden)]
+#[inline]
 pub fn clean_function_name(name: &str) -> &str {
     if let Some(colon) = name.rfind("::") {
         if let Some(colon) = name[..colon].rfind("::") {
@@ -831,6 +865,7 @@ macro_rules! current_file_name {
 
 /// Removes long path prefix to focus on the last parts of the path (and the file name).
 #[doc(hidden)]
+#[inline]
 pub fn short_file_name(name: &str) -> &str {
     // TODO: "foo/bar/src/lib.rs" -> "bar/src/lib.rs"
 
